@@ -2736,7 +2736,7 @@ sub prepare_assets {
 	    $w = $elt->{opts}->{width}
 	      if $elt->{opts}->{width} && $elt->{opts}->{width} < $w;
 
-	    my $res = $hd->( $s, $w, $elt );
+	    my $res = $hd->( $s, elt => $elt, pagewidth => $w );
 	    if ( $res ) {
 		$res->{opts} = { %{ $res->{opts} // {} },
 				 %{ $elt->{opts} // {} } };
@@ -2749,6 +2749,15 @@ sub prepare_assets {
 		  if $config->{debug}->{images};
 		$s->{assets}->{$id} = $res;
 	    }
+	    else {
+		# Substitute alert image.
+		$s->{assets}->{$id} = $res =
+		  { type => "image",
+		    line => $elt->{line},
+		    subtype => "xform",
+		    data => TextLayoutImageElement::alert(60),
+		    opts => { %{$elt->{opts}//{}} } };
+	    }
 
 	    # If the delegate produced an SVG, continue processing.
 	    if ( $res && $res->{type} eq "image" && $res->{subtype} eq "svg" ) {
